﻿using Microsoft.Win32;
using Microsoft.Win32.SafeHandles;
using System;
using System.ComponentModel;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;

namespace NtApi
{
  public sealed class CreateUserProcessResult : IDisposable
  {
    public SafeKernelObjectHandle ProcessHandle
    {
      get; private set;
    }
    public SafeKernelObjectHandle ThreadHandle
    {
      get; private set;
    }
    public SafeFileHandle FileHandle
    {
      get; private set;
    }

    public FileStream GetFileStream(bool writeable)
    {
      if (FileHandle == null)
        throw new ArgumentNullException("FileHandle");

      return new FileStream(FileHandle, writeable ? FileAccess.ReadWrite : FileAccess.Read);
    }

    public SafeKernelObjectHandle SectionHandle { get; private set; }

    public RegistryKey IFEOKeyHandle
    {
      get; private set;
    }

    public SectionImageInformation ImageInfo
    {
      get; private set;
    }

    public ClientId ClientId
    {
      get; private set;
    }

    public int ProcessId
    {
      get
      {
        return ClientId.UniqueProcess.ToInt32();       
      }
    }

    public int ThreadId
    {
      get
      {
        return ClientId.UniqueThread.ToInt32();
      }
    }

    public int Status
    {
      get; private set;
    }

    public bool Success
    {
      get
      {
        return Status == 0;
      }
    }

    public ProcessCreateInfoData CreateInfo
    {
      get; private set;
    }

    public ProcessCreateState CreateState
    {
      get; private set;
    }

    public string GetProcessImageName(bool native_path)
    {
      return NtSystemCalls.GetProcessImageName(ProcessHandle, native_path);
    }

    internal CreateUserProcessResult(IntPtr process_handle, IntPtr thread_handle,
      ProcessCreateInfoData create_info,
      SectionImageInformation image_info, ClientId client_id)
    {
      ProcessHandle = new SafeKernelObjectHandle(process_handle, true);
      ThreadHandle = new SafeKernelObjectHandle(thread_handle, true);
      FileHandle = new SafeFileHandle(create_info.Success.FileHandle, true);
      SectionHandle = new SafeKernelObjectHandle(create_info.Success.SectionHandle, true);
      ImageInfo = image_info;
      ClientId = client_id;
      CreateInfo = create_info;
      CreateState = ProcessCreateState.Success;
    }

    internal CreateUserProcessResult(int status, ProcessCreateInfoData create_info, ProcessCreateState create_state)
    {
      FileHandle = new SafeFileHandle(IntPtr.Zero, false);
      if (create_state == ProcessCreateState.FailOnSectionCreate)
      {
        FileHandle = new SafeFileHandle(create_info.FileHandle, true);
      }
      else if (create_state == ProcessCreateState.FailExeName)
      {
        IFEOKeyHandle = RegistryKey.FromHandle(new SafeRegistryHandle(create_info.IFEOKey, true));
      }
      Status = status;
      CreateInfo = create_info;
      CreateState = create_state;

      ProcessHandle = new SafeKernelObjectHandle(IntPtr.Zero, false);
      ThreadHandle = new SafeKernelObjectHandle(IntPtr.Zero, false);
      SectionHandle = new SafeKernelObjectHandle(IntPtr.Zero, false);
      ImageInfo = new SectionImageInformation();
      ClientId = new ClientId();
    }

    public void Terminate(int exitcode)
    {
      NtSystemCalls.StatusToNtException(NtSystemCalls.NtTerminateProcess(ProcessHandle.DangerousGetHandle(), exitcode));
    }

    public int Resume()
    {
      int previous_count = 0;
      NtSystemCalls.StatusToNtException(NtSystemCalls.NtResumeThread(ThreadHandle.DangerousGetHandle(), out previous_count));
      return previous_count;
    }

    public bool TerminateOnDispose
    {
      get; set;
    }

    #region IDisposable Support
    private bool disposedValue = false; // To detect redundant calls

    private void Dispose(bool disposing)
    {
      if (!disposedValue)
      {
        if (TerminateOnDispose)
        {
          try
          {
            Terminate(1);
          }
          catch (NtException)
          {
          }
        }

        ProcessHandle.Close();
        ThreadHandle.Close();
        FileHandle.Close();
        SectionHandle.Close();
        if (IFEOKeyHandle != null)
        {
          IFEOKeyHandle.Close();
        }
        disposedValue = true;
      }
    }

    ~CreateUserProcessResult()
    {
      Dispose(false);
    }

    public void Dispose()
    {
      Dispose(true);
      GC.SuppressFinalize(this);
    }
    #endregion
  }


}
